#!/bin/bash

# ## Begin configuration secton.

# ## Configure paths
# ## These paths are correct for programs installed according to john-quensen.com tutorials.
# ## Required programs are RDPTools, FunGene_pipeline, usearch version 8, FastTree, and HMMER3.
# RDPToolsDir=/usr/local/RDPTools
# usearch=/usr/local/bin/usearch8.1
# hmmerDir=/usr/bin/	
# FastTree=/usr/bin/fasttree

## For HPCC
## These paths are correct for MSU's HPCC
RDPToolsDir=/mnt/research/rdp/public/RDPTools
hmmerDir=/mnt/research/rdp/public/thirdParty/hmmer-3.1b2-linux-intel-ia32/binaries
module load USEARCH
module load FastTree
usearch=usearch
FastTree=FastTree

# ## Gene specific parameters:
# ## HMMER3 model for local installation:
# hmmfile=/usr/local/fungene_pipeline/resources/nifh/model.hmm

# ## HMMER3 model for MSU's HPCC:
hmmfile=/mnt/research/rdp/public/fungene_pipeline/resources/nifh/model.hmm

# Framebot reference file
gene_ref_file=nifh_prot_ref_polyprimers.fasta

# Set java heap size
JAVA_HEAP_SIZE=2g

# ## End configuration secton.

# Run uchime on pooled samples
# First, dereplicate, add size info & sort
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar derep -u -s -o derep.fa all_seqs.ids all_seqs.samples trimmed_seqs/*.fastq
## Remove spaces from fasta IDs; else uchime will not run
cat derep.fa | sed 's/ //g' > derep_no_space.fa 
$usearch -uchime_denovo derep_no_space.fa -nonchimeras nc.fasta # Run uchime in denovo mode
# Put spaces back into fasta IDs; else refresh-mappings will not run
cat nc.fasta | sed 's/length/  length/' > nc_with_space.fasta 
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar refresh-mappings nc_with_space.fasta all_seqs.ids all_seqs.samples filtered_all_seq.ids filtered_all_seq.samples
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar explode-mappings -o explode-mappings filtered_all_seq.ids filtered_all_seq.samples nc_with_space.fasta

# Run FrameBot on sequences in directory explode-mappings.
# Move results into directory fb_out
mkdir fb_out
for f in $(ls explode-mappings/*.fasta)
do
	d=${f/explode-mappings\//}
	d=${d/.fasta/}
	java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/FrameBot.jar framebot -N -o $d $RDPToolsDir/Framebot/refset/$gene_ref_file $f
	mv $d*.* fb_out
done

# Copy corrected protein sequences to directory fb_corr_prot.
mkdir fb_corr_prot
cp fb_out/*corr_prot.fasta fb_corr_prot

# Align the framebot corrected proteins sequences.
mkdir hmm_aligned
for f in $(ls fb_corr_prot/*.fasta)
do
	d=${f/fb_corr_prot\//}
	d=${d/_corr_prot/}
	d=${d/.fasta/}
	$hmmerDir/hmmalign -o hmm_aligned/$d.stk $hmmfile $f
done

# Merge the alignments.
mkdir merged_alignment
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/AlignmentTools.jar alignment-merger hmm_aligned merged_alignment/merged_aligned.fasta

## Cluster, 3 steps.
mkdir cluster
cd cluster
# 1) Dereplicate
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar derep -m '#=GC RF' -o derep.fa all_seqs.ids all_seqs.samples ../hmm_aligned/*.stk

# 2) Calculate the distance matrix.
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar dmatrix --id-mapping all_seqs.ids --in derep.fa --outfile derep_matrix.bin -l 50 --dist-cutoff 0.1

# 3) Cluster
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar cluster --dist-file derep_matrix.bin --id-mapping all_seqs.ids --sample-mapping all_seqs.samples --method complete --outfile all_seq_complete.clust

# Convert cluster file to biom format.
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar cluster-to-biom all_seq_complete.clust 0.03 > all_seq_complete.clust.biom

# Get representative sequences.
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar rep-seqs -c --id-mapping all_seqs.ids --one-rep-per-otu all_seq_complete.clust 0.03 ../merged_alignment/merged_aligned.fasta

# Unalign representative sequences and remove descriptions.
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar to-unaligned-fasta all_seq_complete.clust_rep_seqs.fasta | cut -f1 -d ' ' > unaligned_short_names.fasta

# Tree the representative sequences.
# First, extract the comparable (model) positions for tree building.
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/Clustering.jar derep -f -o all_seq_complete.clust_rep_seqs_model_only.fasta rep_seqs.ids rep_seqs.sample all_seq_complete.clust_rep_seqs.fasta

$FastTree < all_seq_complete.clust_rep_seqs_model_only.fasta > my_expt_tree.nwk

# Match FrameBot matches to machine names of representative sequences.
cd ../
mkdir taxa
cp fb_out/*.txt taxa
rm taxa/*failed*.txt
cat taxa/*.txt > matches.txt
grep "STATS" matches.txt | cut -f2,3,4,5,6 | cut -f5,6 -d "|" | sed 's/|/\t/' > match_taxa_machine_names.txt

# Match cluster numbers to machine names of representative sequences.
grep ">" cluster/all_seq_complete.clust_rep_seqs.fasta | cut -f1 -d ","  | sed 's/>//' | sed 's/seq_id=//' | sed 's/  /\t/'  > match_cluster_machine_name.txt

# Package results for phyloseq.
tar -czf results4phyloseq.tar.gz cluster/all_seq_complete.clust cluster/all_seq_complete.clust.biom cluster/my_expt_tree.nwk match_taxa_machine_names.txt match_cluster_machine_name.txt cluster/unaligned_short_names.fasta

# Make stat reports
java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/FrameBot.jar stat -t matrix -e 80 -i filtered_all_seq.ids -s filtered_all_seq.samples matches.txt R_table_80.txt

java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/FrameBot.jar stat -t hist -i filtered_all_seq.ids -s filtered_all_seq.samples matches.txt hist_report.txt

java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/FrameBot.jar stat -t stat -i filtered_all_seq.ids -s filtered_all_seq.samples matches.txt stat_report.txt

java -Xmx$JAVA_HEAP_SIZE -jar $RDPToolsDir/FrameBot.jar stat -t summary -i filtered_all_seq.ids -s filtered_all_seq.samples matches.txt summary_report.txt

## Package stat reports
tar -czf stat_reports.tar.gz R_table_80.txt hist_report.txt stat_report.txt summary_report.txt

